import app/web
import gleam/bytes_tree
import gleam/http/request.{type Request}
import gleam/http/response.{type Response}
import gleam/option.{None}
import gleam/result
import gleam/string
import mist.{type Connection, type ResponseData}

pub fn handle_request(
  request: Request(Connection),
  ctx: web.Context,
) -> Response(ResponseData) {
  case request.path_segments(request) {
    [] -> serve_frontend()
    ["static", "frontend.wasm"] -> serve_wasm(ctx)
    _ -> response.set_body(response.new(404), mist.Bytes(bytes_tree.new()))
  }
}

const game_body = "
<html lang=\"en\">

<head>
    <meta charset=\"utf-8\">
    <title>TITLE</title>
    <style>
        html,
        body,
        canvas {
            margin: 0px;
            padding: 0px;
            width: 100%;
            height: 100%;
            overflow: hidden;
            position: absolute;
            background: black;
            z-index: 0;
        }
    </style>
</head>

<body>
    <canvas id=\"glcanvas\" tabindex='1'></canvas>
    <!-- Minified and statically hosted version of https://github.com/not-fl3/macroquad/blob/master/js/mq_js_bundle.js -->
    <script src=\"https://not-fl3.github.io/miniquad-samples/mq_js_bundle.js\"></script>
    <script>load(\"static/frontend.wasm\");</script>
</body>

</html>
"

fn serve_frontend() -> Response(ResponseData) {
  response.new(200)
  |> response.set_body(mist.Bytes(bytes_tree.from_string(game_body)))
  |> response.set_header("content-type", "text/html")
}

fn serve_wasm(ctx: web.Context) -> Response(ResponseData) {
  let wasm_path = string.join([ctx.config.static_path, "frontend.wasm"], "/")
  mist.send_file(wasm_path, offset: 0, limit: None)
  |> result.map(fn(file) {
    response.new(200)
    |> response.prepend_header("content-type", "application/wasm")
    |> response.set_body(file)
  })
  |> result.lazy_unwrap(fn() {
    response.new(404)
    |> response.set_body(mist.Bytes(bytes_tree.new()))
  })
}
